/* 
   The purpose of this function is provide an easy, software
   based method of generating random numbers.  The method 
   by which this is accomplished is described below.

   Since they are based on conditions of the system, if
   used on a *nix system they yeild very random and
   unpredictable results due to swapping and other odd
   things (possibly even temperature)
   
   Since we measure the time of execution of a program
   in the highest precision measurable per system (normally
   microseconds), and use it as a large factor in our
   random number algorithm, we get affected easily by any-
   thing.
   
     -- rkeene [310719991500] keene@indiana.nrlssc.navy.mil

*/
extern unsigned long generate_random_number (unsigned long max);
extern unsigned long GetClicks (void);
