void cd_eject (int cdrom_fd);
void cd_stop  (int cdrom_fd);
void cd_play  (int cdrom_fd, int starttrack);
void cd_playtrack  (int cdrom_fd, int track);
void cd_setvolume  (int cdrom_fd, unsigned char volume);
int cd_num_tracks (int cdrom_fd, int start);
int cd_isplaying (int cdrom_fd);
int cd_currenttrack (int cdrom_fd);
int cd_getvolume (int cdrom_fd);
int open_cdrom (char *cdromfile);
