#include "version.h"

#define CDPLAY_NOACTION		-1
#define CDPLAY_EJECT 		1
#define CDPLAY_DISPLAY		2
#define CDPLAY_SETVOL		3
#define CDPLAY_RAWPLAY		5

#define CDPLAY_LOOP_PLAY	cd_playtrack
#define CDPLAY_CHECK_DELAY	70000
